# this python script performs the same calculation as example_ra.ipynb

import json
import argparse
import os

from aivis_engine_v2_ra_sdk_python import (
    ResponseAnalysisSetup,
    ResponseAnalysisData,
    ResponseAnalysis,
    EngineLogger,
)

# ---------------------------------------------------------------------------------
# Data location

INPUT_DATA_FILE = "train_ra.csv"
TARGET_COLUMN_ID = "TARGET"


# Input and output folder specified via args
def from_args():
    parser = argparse.ArgumentParser(
        description="Example project for response analysis"
    )
    # data is expected in input folder
    parser.add_argument(
        "--input", "-i", type=str, nargs=1, required=True, help="Path to input folder"
    )
    # output files will be written to output folder
    parser.add_argument(
        "--output", "-o", type=str, nargs=1, required=True, help="Path to output folder"
    )
    args = parser.parse_args()
    # create output folder if not existing
    if not os.path.exists(args.output[0]):
        os.mkdir(args.output[0])
    return {
        "input_folder": args.input[0],
        "output_folder": args.output[0],
    }


# ---------------------------------------------------------------------------------
# Logger and Threads


class Logger(EngineLogger):
    def log(self, level, thread, module, message):
        min_logging_level = 3
        if level <= min_logging_level:
            print("\t... %s" % message)


def setup_logger_and_threads():
    print("Initializing ResponseAnalysisSetup: logging and threads ...")
    ResponseAnalysisSetup.register_logger(Logger())
    max_threads = 0  # number of threads will be chosen automatically with 0
    ResponseAnalysisSetup.init_thread_count(max_threads)
    print("\t... done!")
    return


# ---------------------------------------------------------------------------------
# Reading and Writing


def read_json(filename):
    print("\t... reading file '%s' ..." % filename)
    with open(filename, "r") as file:
        return json.loads(file.read())


def write_json(filename, obj):
    print("\t... writing file '%s' ..." % filename)
    with open(filename, "w") as file:
        file.write(json.dumps(obj, indent=2))
    return


# ---------------------------------------------------------------------------------
# MAIN


def main():
    folder_config = from_args()
    setup_logger_and_threads()

    print("Loading input data ...")
    analysis_data = ResponseAnalysisData.create()

    string_columns = [
        "status_of_existing_checking_account",
        "purpose",
        "savings_account",
        "credit_history",
        "marital_status_and_sex",
        "other_debtors",
        "property",
        "other_installment_plans",
        "housing",
        "job",
        "telephone",
        "foreign_worker",
    ]

    files_filter = {"includeFiles": [INPUT_DATA_FILE]}
    data_types = {"stringColumns": string_columns}
    files_reader_config = json.dumps(
        {
            "folder": folder_config["input_folder"],
            "filesFilter": files_filter,
            "dataTypes": data_types,
        }
    )

    analysis_data.read_files(files_reader_config)

    # print some record to illustrate type DtoFloatCell
    print("Example record!")
    some_column_id = "credit_amount"
    print("\t... Column id:", some_column_id)
    some_column_cells = analysis_data.get_float_column(some_column_id)
    print("\t... Column length:", len(some_column_cells))
    print("\t... Column entry number 42:", some_column_cells[42])
    print(
        "\t... Column entry number 42 row_id:",
        some_column_cells[42].id,
    )
    print("\t... Column entry number 42 value:", some_column_cells[42].value)

    # ---- ANALYZING ----

    print("Creating analysis ...")
    # build analysis config - for more options see documentation
    analysis_config = json.dumps(
        {
            "dataFilter": {"excludeColumns": ["telephone"]},
            "target": {
                "kpi": TARGET_COLUMN_ID,
                "interest": "HIGH_KPI",
            },
            "strategy": {
                "minimalFractionOfRecords": 0.05,
            },
        }
    )

    # create analysis
    analysis = ResponseAnalysis.create(analysis_data, analysis_config)
    print("\t... done!")

    print("Save analysis report ...")
    report = analysis.get_report()
    write_json(
        os.path.join(folder_config["output_folder"], "analysis-report.json"),
        json.loads(report),
    )

    print("\t... done!")

    print("Generating report ...")

    # ---- Toolbox HTML Report ----
    from aivis_engine_v2_toolbox.api import build_response_analysis_report

    build_response_analysis_report(
        {
            "title": "Aivis Engine V2 - Response Analysis Example: German Credit Health",
            "archive": {
                "files": [os.path.join(folder_config["input_folder"], INPUT_DATA_FILE)]
            },
            "analysisReportFile": os.path.join(
                folder_config["output_folder"], "analysis-report.json"
            ),
            "outputFile": os.path.join(
                folder_config["output_folder"],
                "aivis-engine-v2-response-analysis-example.html",
            ),
        }
    )
    print("ALL DONE!")


# run main function
main()
